from django.contrib import admin
from system_globals.models import SystemGlobal

class SystemGlobalAdmin(admin.ModelAdmin):
    list_display = ('var_name', 'value', 'description',)

admin.site.register(SystemGlobal, SystemGlobalAdmin)




from django.conf import settings
from django.core.cache import cache
from django.db import connection
from django.test import TestCase
from system_globals.models import SystemGlobal


class SystemGlobalTest(TestCase):

    def setUp(self):
        self.sg1 = SystemGlobal.objects.create(var_name='testing_one', value='test value')
        self.sg2 = SystemGlobal.objects.create(var_name='TESTING_TWO', value='\t t')
        self.sg3 = SystemGlobal.objects.create(var_name='testing_three', value='3.0 ')

    def tearDown(self):
        cache.clear()
        self.sg1.delete()
        self.sg2.delete()
        self.sg3.delete()

    def test_as_dict_with_prefix(self):
        dictionary = SystemGlobal.objects.as_dict(prefix='testing_')
        self.assertTrue('one' in dictionary)
        self.assertTrue('TWO' in dictionary)
        self.assertTrue('three' in dictionary)

        self.assertEqual(dictionary['one'], 'test value')
        self.assertEqual(dictionary['TWO'], True)
        self.assertEqual(dictionary['three'], 3.0)

        dictionary = SystemGlobal.objects.as_dict(prefix='teSTinG_')
        self.assertTrue('one' in dictionary)
        self.assertTrue('TWO' in dictionary)
        self.assertTrue('three' in dictionary)

        self.assertEqual(dictionary['one'], 'test value')
        self.assertEqual(dictionary['TWO'], True)
        self.assertEqual(dictionary['three'], 3.0)

        dictionary = SystemGlobal.objects.as_dict(prefix='non_existent')
        self.assertEqual(dictionary, {})

    def test_as_dict_without_prefix(self):
        dictionary = SystemGlobal.objects.as_dict()
        self.assertTrue('testing_one' in dictionary)
        self.assertTrue('TESTING_TWO' in dictionary)
        self.assertTrue('testing_three' in dictionary)

        self.assertEqual(dictionary['testing_one'], 'test value')
        self.assertEqual(dictionary['TESTING_TWO'], True)
        self.assertEqual(dictionary['testing_three'], 3.0)

    def test_as_dict_without_coerce(self):
        dictionary = SystemGlobal.objects.as_dict(coerce=False)
        self.assertTrue('testing_one' in dictionary)
        self.assertTrue('TESTING_TWO' in dictionary)
        self.assertTrue('testing_three' in dictionary)

        self.assertEqual(dictionary['testing_one'], 'test value')
        self.assertEqual(dictionary['TESTING_TWO'], '\t t')
        self.assertEqual(dictionary['testing_three'], '3.0 ')

    def test_as_dict_with_to_lower(self):
        dictionary = SystemGlobal.objects.as_dict(to_lower=True)
        self.assertTrue('testing_one' in dictionary)
        self.assertTrue('testing_two' in dictionary)
        self.assertTrue('testing_three' in dictionary)

        self.assertEqual(dictionary['testing_one'], 'test value')
        self.assertEqual(dictionary['testing_two'], True)
        self.assertEqual(dictionary['testing_three'], 3.0)

    def test_as_dict_using_cache(self):
        # Storing the original DEBUG value
        original_debug = settings.DEBUG

        # The Django docs says that DEBUG should be set to True for store queries
        # https://docs.djangoproject.com/en/dev/faq/models/#how-can-i-see-the-raw-sql-queries-django-is-running
        settings.DEBUG = True
        SystemGlobal.objects.as_dict()
        SystemGlobal.objects.as_dict()
        SystemGlobal.objects.as_dict()
        SystemGlobal.objects.as_dict(prefix='Test01')
        SystemGlobal.objects.as_dict(prefix='Test02')
        SystemGlobal.objects.as_dict(prefix='Test03')
        SystemGlobal.objects.as_dict(prefix='Test04', to_lower=True)
        SystemGlobal.objects.as_dict(prefix='Test05', to_lower=True)
        SystemGlobal.objects.as_dict(prefix='Test06', to_lower=True)
        SystemGlobal.objects.as_dict(prefix='Test07', to_lower=True, coerce=False)
        SystemGlobal.objects.as_dict(prefix='Test08', to_lower=True, coerce=False)
        SystemGlobal.objects.as_dict(prefix='Test09', to_lower=True, coerce=False)
        SystemGlobal.objects.as_dict(prefix='Test10', coerce=False)
        SystemGlobal.objects.as_dict(prefix='Test11', coerce=False)
        SystemGlobal.objects.as_dict(prefix='Test12', coerce=False)

        self.assertEqual(len(connection.queries), 0)

        # Restoring the original DEBUG value
        settings.DEBUG = original_debug

    def test_as_dict_after_set_a_integer(self):
        SystemGlobal.objects.set('TESTING_ONE', 1010)
        SystemGlobal.objects.as_dict()

    def test_as_dict_after_set_a_float(self):
        SystemGlobal.objects.set('TESTING_ONE', 3.1416)
        SystemGlobal.objects.as_dict()

    def test_as_dict_after_set_a_boolean(self):
        SystemGlobal.objects.set('TESTING_ONE', True)
        SystemGlobal.objects.as_dict()

    def test_set(self):
        SystemGlobal.objects.set('TESTING_TWO', ' SystemGlobals Testing\n')
        value = SystemGlobal.objects.get_value('TESTING_TWO')
        self.assertEquals(value, 'SystemGlobals Testing')

        SystemGlobal.objects.set('TESTING_four', ' SystemGlobals new Testing\n')
        value = SystemGlobal.objects.get_value('TESTING_four')
        self.assertEquals(value, 'SystemGlobals new Testing')

        SystemGlobal.objects.set('TESTING_five', 123)
        value = SystemGlobal.objects.get_value('TESTING_five')
        self.assertEquals(value, 123)

        # TearDown
        SystemGlobal.objects.filter(var_name='TESTING_four').delete()
        SystemGlobal.objects.filter(var_name='TESTING_five').delete()

    def test_get_value(self):
        value = SystemGlobal.objects.get_value('TESTING_TWO')
        self.assertEquals(value, True)

        value = SystemGlobal.objects.get_value('testing_three')
        self.assertEquals(value, 3.0)

    def test_coerce(self):
        """
        Testing SystemGlobal.coerce static method.
        """

        # Float
        coerced_value = SystemGlobal.coerce('1.0')
        self.assertEquals(coerced_value, 1.0)

        coerced_value = SystemGlobal.coerce('3.33')
        self.assertEquals(coerced_value, 3.33)

        # Int
        coerced_value = SystemGlobal.coerce(' 3 ')
        self.assertEquals(coerced_value, 3)

        # True
        coerced_value = SystemGlobal.coerce(' true ')
        self.assertEquals(coerced_value, True)

        coerced_value = SystemGlobal.coerce('\n yEs ')
        self.assertEquals(coerced_value, True)

        coerced_value = SystemGlobal.coerce('\n Y \t')
        self.assertEquals(coerced_value, True)

        coerced_value = SystemGlobal.coerce('t')
        self.assertEquals(coerced_value, True)

        # False
        coerced_value = SystemGlobal.coerce(' faLse ')
        self.assertEquals(coerced_value, False)

        coerced_value = SystemGlobal.coerce(' no ')
        self.assertEquals(coerced_value, False)

        coerced_value = SystemGlobal.coerce('\t\tn\n\n')
        self.assertEquals(coerced_value, False)

        coerced_value = SystemGlobal.coerce('F')
        self.assertEquals(coerced_value, False)

        # String
        coerced_value = SystemGlobal.coerce(' value ')
        self.assertEquals(coerced_value, 'value')

    def test_stale_cache(self):
        self.assertEqual(SystemGlobal.objects.get_value('testing_one'), 'test value')

        one = SystemGlobal.objects.get(var_name='testing_one')
        one.value = 'test2 value'
        one.save()
        self.assertEqual(SystemGlobal.objects.get_value('testing_one'), 'test2 value')

        SystemGlobal.objects.filter(var_name='testing_one').update(value='test3 value')
        self.assertEqual(SystemGlobal.objects.get_value('testing_one'), 'test3 value')

    def test_system_global_queryset(self):
        """ Test SystemGlobal.SystemGlobalQuerySet """
        updated_rows = SystemGlobal.objects.all().update(value='test value')
        total_globals = SystemGlobal.objects.count()
        self.assertEqual(updated_rows, total_globals)

        # extra check to make sure SystemGlobal are saving values in db
        SystemGlobal.objects.set('SAVED', ' SystemGlobals Testing')
        self.assertTrue(SystemGlobal.objects.get_value('SAVED'))

        new_total_globals = SystemGlobal.objects.count()
        self.assertEqual(new_total_globals, total_globals + 1)



import re

import django
from django.core.cache import cache
from django.db import models
from django.db.models.signals import post_save
from django.utils.translation import ugettext_lazy as _


class SystemGlobalQuerySet(models.query.QuerySet):
    """QuerySet implementation that refreshes the cache on update."""
    def update(self, **kwargs):
        updated_rows = super(SystemGlobalQuerySet, self).update(**kwargs)

        if 'value' in kwargs:
            for var_name in self.values_list('var_name', flat=True):
                SystemGlobal.objects._update_cache(var_name, kwargs['value'])

        return updated_rows


class SytemGlobalManager(models.Manager):
    """
    Manager for the SystemGlobal class, with
    helper methods and cache management.
    """
    def get_queryset(self):
        return SystemGlobalQuerySet(SystemGlobal)

    if django.VERSION < (1, 6):
        get_query_set = get_queryset

    def get_value(self, var_name):
        """
        Given a variable name, returns SystemGlobal coerced value.
        """
        cached_dict = self._get_dict()
        if var_name in cached_dict:
            value = cached_dict.get(var_name)
            return SystemGlobal.coerce(str(value))

        raise SystemGlobal.DoesNotExist('%s was not set in SystemGlobals.' % var_name)

    def as_dict(self, prefix='', to_lower=False, coerce=True):
        """
        Given a prefix, searches all the SystemGlobals which starts with that
        prefix, and create a dictionary where the keys haven't that prefix,
        otherwise gets all the SystemGlobals. The prefix is case-insensitive.

        If to_lower parameter is set to True the dict keys would be lowercase.
        """
        cached_dict = self._get_dict()

        result_dict = {}
        prefix_len = len(prefix)
        for key in cached_dict.keys():
            if key.lower().startswith(prefix.lower()):
                value = str(cached_dict[key])
                key = key.lower() if to_lower else key
                key = key[prefix_len:]
                result_dict[key] = SystemGlobal.coerce(value) if coerce else value

        return result_dict

    def set(self, var_name, value):
        """
        Make a atomic query for set a new value for a existent SystemGlobal, or
        create the new variable and set the new value.
        """
        updated_rows = self.filter(var_name=var_name).update(value=value)
        if not updated_rows:
            self.create(var_name=var_name, value=value)

        self._update_cache(var_name, value)

    def _update_cache(self, var_name, value):
        cached_dict = self._get_dict()
        cached_dict[var_name] = value
        cache.set('SystemGlobals', cached_dict)

    def _get_dict(self):
        cached_dict = cache.get('SystemGlobals')
        if cached_dict is not None:
            return cached_dict
        return self._update_dict_from_db()

    def _update_dict_from_db(self):
        system_globals = self.all().values('var_name', 'value')
        dictionary = {}
        for system_global in system_globals:
            var_name = system_global['var_name']
            value = system_global['value']
            dictionary[var_name] = value

        dictionary = dict([(str(k), v) for k, v in dictionary.items()])
        cache.set('SystemGlobals', dictionary)
        return dictionary


class SystemGlobal(models.Model):
    """
    This model stores all the Globals that could be used in a System.
    """
    var_name = models.CharField(_(u'variable name'), max_length=100, unique=True)
    value = models.TextField(_(u'value'))
    description = models.TextField(_(u'description'), blank=True)

    objects = SytemGlobalManager()

    class Meta:
        verbose_name_plural = _(u'System Globals')

    def __unicode__(self):
        return "%s = %s" % (self.var_name, self.value)

    def coerced_value(self):
        return self.coerce(self.value)

    @staticmethod
    def coerce(raw_value):
        """
        Do some friendly type conversion.
        """
        val = raw_value.strip()
        if re.match(r'^(\d*)\.\d+$', val):  # float
            return float(val)
        elif re.match(r'^\d+$', val):  # int
            return int(val)
        elif val.lower() in ('true', 't', 'yes', 'y'):
            return True
        elif val.lower() in ('false', 'f', 'no', 'n'):
            return False
        else:   # must be a string
            return val


def update_cached_global(sender, instance, **kwargs):
    SystemGlobal.objects._update_cache(instance.var_name, instance.value)

post_save.connect(update_cached_global, sender=SystemGlobal, dispatch_uid="update_cached_global")



#!/usr/bin/env python
import os
from system_globals import version
from setuptools import setup

def get_packages():
    # setuptools can't do the job :(
    packages = []
    for root, dirnames, filenames in os.walk('system_globals'):
        if '__init__.py' in filenames:
            packages.append(".".join(os.path.split(root)).strip("."))

    return packages

setup(name='django-system-globals',
    version=version,
    description='A Django App to manage any system globals from a DB table.',
    author=u'Andres Torres Marroquin',
    author_email='andres.torres.marroquin@gmail.com',
    url='https://github.com/andres-torres-marroquin/django-system-globals',
    packages=get_packages(),
)
